<?php

/**
 * Projects Page
 *
 * Callback that is invoked when the user points the brower to
 * hosting/projects. It displays a nice tabulated list of projects
 * and cool things you can do with them.
 *
 * @TODO: Should we make this a list of teasers?  Hard to cram everything in
 * a table.
 */
function devshop_projects_projects_page() {

  $header = array(
    'Name',
    'Profile',
    'Version',
    '',
    'Git URL',
    'Dev Site',
  );

  $r = db_query("SELECT hdp.* FROM {hosting_devshop_project} hdp LEFT JOIN {node} n ON hdp.nid=n.nid WHERE n.status=1");
  $rows = array();

  while(($proj = db_fetch_object($r))) {
    $node = node_load($proj->nid);
    if (!empty($node->project_objects['platform'])){
      $platform_node = node_load(key($node->project_objects['platform']));
    }

    $row = array();
    $link_options = array('attributes' => array('target' => '_blank'));

    // Link to Project page
    $row[] = '<strong>' . l($node->title, "node/$proj->nid") . '</strong>';

    // Install Profile
    $row[] = $node->install_profile;

    // Drupal Version
    $row[] = $platform_node->release->version;

    // Number of environments
    $num = db_result(db_query("SELECT COUNT(*) FROM {hosting_devshop_project_object} WHERE project_nid = %d AND object_type='site'", $node->nid));
    $row[] = format_plural($num, t('1 site'), t('!num sites', array('!num' => $num)));



    // Git URL
    $row[] = strtr("<input type='textfield' value='!url' onclick='this.select();'/>", array('!url' => $node->git_url));

    // Link to Dev Site
    $dev_site_url = url("http://dev." . $node->base_url, array('absolute' => TRUE));
    $row[] = l($dev_site_url, $dev_site_url, $link_options);


    $rows[$proj->nid] = $row;
  }

  //Call hooks for alter informacion
  $data = module_invoke_all('devshop_projects_page', $rows, $header);


  $output = theme('table', $data['header'], $data['rows'], array('class' => 'hosting-table'));
  $output .= l(t('Create a new project?'), 'node/add/project');
  return $output;
}

/**
 * Implementation of hook_view().
 *
 * Project Page Display
 */
function devshop_projects_view($node, $teaser = FALSE, $page = FALSE) {

  modalframe_parent_js();

  $node->content['info'] = array(
    '#prefix' => '<div id="hosting-platform-info">',
    '#suffix' => '</div>'
  );

  $node->content['info']['code_path'] = array(
    '#type' => 'item',
    '#title' => t('Code path'),
    '#value' => filter_xss($node->code_path),
    '#weight' => -8
  );

  $url = 'http://dev.' . $node->base_url;
  $node->content['info']['base_url'] = array(
    '#type' => 'item',
    '#title' => t('Dev Site'),
    '#value' => l($url, $url, array('attributes' => array('target' => '_blank'))),
    '#weight' => -10
  );

  $node->content['info']['git_url'] = array(
    '#type' => 'item',
    '#title' => t('Git URL'),
    '#value' => strtr("<input type='textfield' value='!url' onclick='this.select();'/>", array('!url' => $node->git_url)),
    '#weight' => -10
  );

  if (!empty($node->git_branches)){
    $node->content['info']['git_branches'] = array(
      '#type' => 'item',
      '#title' => t('Remote Branches'),
      '#value' => implode($node->git_branches, ' '),
      '#weight' => -8
    );
  }

  if (!empty($node->install_profile)){
    $node->content['info']['install_profile'] = array(
      '#type' => 'item',
      '#title' => t('Install profile'),
      '#value' => ($node->install_profile),
      '#weight' => -8
    );
  }



  $rows = array();
  if (isset($node->project_objects['site'])) {
    foreach($node->project_objects['site'] as $nid => $env) {
      $site = node_load($nid);
      $platform = node_load($site->platform);
      if ($site->site_status == -2 && $platform->platform_status == -2) {
        continue;
      }
      $row = array();
      if ($site->site_status != -2) {
        $url = "http://$site->title";
        $row[] = l($url, $url, array('attributes' => array('target' => '_blank')));
        $row[] = l('Site Dashboard', "node/$site->nid");
      }
      else {
        // Site if deleted, just show the platform name
        $row[] = $platform->title;
        $row[] = '';
      }
      $row[] = l('Platform Dashboard', "node/$site->platform");

      if (module_exists('devshop_log')){
        $row[] = l('Commit Log', "node/$site->nid/logs/commits");
      }
      if (module_exists('hosting_logs')){
        $row[] = l('Error Log', "node/$site->nid/logs/errors");
      }
      if (module_exists('hosting_filemanager')){
        $row[] = l('Files', "node/$site->nid/files/platform");
      }
      $rows[] = $row;
    }
    $header = array();
    $table = theme('table', $header, $rows);

    $node->content['sites'] = array(
      '#type' => 'fieldset',
      '#title' => t('Project Environments'),
      '#weight' => 12,
      '#attributes' => array(
        'class' => 'project-environments',
      ),
    );
    $node->content['sites']['table'] = array(
      '#type' => 'item',
      '#value' => $table,
    );
  }

  //Tasks
  $node->content['tasks_view'] = array(
    '#type' => 'item',
    //'#value' => devshop_projects_hosting_task_table($node),
    '#value' => hosting_task_table($node),
    '#prefix' => '<div id="hosting-task-list">',
    '#suffix' => '</div>',
    '#weight' => 10
  );
  $settings['hostingTaskRefresh'] = array(
    'nid' => $node->nid,
    'changed' => $node->changed,
  );

  drupal_add_js($settings, 'setting');
  drupal_add_js(drupal_get_path('module','hosting_task') . '/hosting_task.js');
  drupal_add_css(drupal_get_path('module', 'devshop_projects') . '/inc/environments.css');

  // MAIN DISPLAY
  $node->content['devshop'] = array(
    '#type' => 'fieldset',
    '#weight' => 100,
  );
  return $node;
}
